//
// Copyright (C) 2025 Carl Klemm <carl@uvos.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
// the Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

#include <iostream>
#include <filesystem>
#include <eisgenerator/translators.h>
#include <kisstype/type.h>

#include "csvBackend.h"
#include "common.h"

int main(int argc, char** argv)
{
	if(argc < 4)
	{
		std::cerr<<"Usage: "<<(argc > 0 ? argv[0] : "NULL")<<" [OUTDIR] [MODEL] [FILENAME(S)]\n";
		return 1;
	}

	std::filesystem::path outDir(argv[1]);
	if(!checkDir(outDir))
		return 1;

	try
	{
		for(int i = 3; i < argc; ++i)
		{
			try
			{
				io::CSVReader<3, io::trim_chars<' '>, io::no_quote_escape<'\t'>> csvFile(argv[i]);
				if(!csvFile.next_line())
					return 1;

				csvFile.read_header(io::ignore_extra_column, "Impedance Real part", "Impedance Imaginary Part", "Frequency [Hz]");

				std::string frequency;
				std::string real;
				std::string imaginary;
				ModelData data;
				data.modelStr = eis::relaxisToEis(argv[2]);
				data.id = std::to_string(i-3);
				while(csvFile.read_row(real, imaginary, frequency))
					data.data.push_back(eis::DataPoint(std::complex<fvalue>(std::stod(real), std::stod(imaginary)), std::stod(frequency)*2*M_PI));

				saveData("maryam", data, outDir, argv[i]);
			}
			catch(const io::error::missing_column_in_header& err)
			{
				std::cerr<<"Could not read file "<<err.what()<<std::endl;
			}
		}
	}
	catch (const std::invalid_argument& err)
	{
		std::cerr<<"Could not read file "<<err.what();
		return 1;
	}
	return 0;
}
